% For use with Matlab
% Copyright (C) 2020 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza. "Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control". Applied Sicences, 2020.
% The "uSelf-FLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza.
% Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control Rules.
% Applied Sciences, 2020, 10(17), 5836.
% DOI: https://doi.org/10.3390/app10175836
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [ ] = Plot_Evolve_Parameters (Evolve_Parameters,delta_dif_controller)
%% Description
% This function does plots of some important parameters on the evolving stage

%% Function inputs
% Evolve_Parameterts - Struct with some importante parameters on the evolving stage to do plots

%% Function output
% several plots

%% Flag Add
figure
plot(Evolve_Parameters.Add) % Plot of the flag Add - indicates when new fuzzy rule was added
labels{1} = 'New rule was added'; % Save legend

hold on
plot(Evolve_Parameters.del) % Plot of the flag Delete - indicates when new fuzzy rule was added
labels{2} = 'New rule was delete'; % Save legend

%% Number of rules per input variable
for j=1:size(Evolve_Parameters.nRules,2)
    hold on
    plot(Evolve_Parameters.nRules{j})
    labels{j+2} = strcat('Number of rules of x_', num2str(j)); % Save legend
    AddRules(j,:)=Evolve_Parameters.nRules{j};% Just to save to make plots
    
end
legend(labels) % Figure legends
title('Evolution of the fuzzy rules') % Figure title

%% Evolution of Criterion 1
figure
plot(delta_dif_controller)
title('Evolution of Criterion 1') % Figure title

%% Evolution of the center of the antecedents membership functions and of the consequents
kk=1;
kkk=1;

for j=1:size(Evolve_Parameters.nRules,2)
    
    clear antecedents consequents
    
    antecedents = zeros(max(Evolve_Parameters.nRules{:,j}),size(Evolve_Parameters.MFs,2));
    consequents = antecedents;
    
    % change of variables
    for k=1:size(Evolve_Parameters.MFs,2)
        aux= Evolve_Parameters.MFs{j,k,:};
        for i=1:size(aux,1)
            antecedents(i,k) = aux(i,2);
            consequents(i,k) = aux(i,4);
        end
        
    end
    
    % Plot of the center of the antecedents membership functions
    figure
    for i=1:size(antecedents,1)
        hold on
        plot(antecedents(i,:))
        labels{i} = strcat('MF  ', num2str(i)); % Save legend
        AntParameters(kk,:) = antecedents(i,:);% Just to save to make plots
        kk=kk+1;
    end
    legend(labels) % Figure legends
    title(sprintf('Evolution of the antecendents of x_{%d}', j)) % Figure title
    
    % Plots of the consequents
    figure
    for i=1:size(consequents,1)
        hold on
        plot(consequents(i,:))
        labels{i} = strcat('Rule ', num2str(i)); % Save legend
        ConsParameters(kkk,:) = consequents(i,:);% Just to save to make plots
        kkk=kkk+1;
    end
    legend(labels) % Figure legends
    title(sprintf('Evolution of the consequents of x_{%d}', j)) % Figure title
end

end
